@@ -1,50 +1,55 @@
 #' Differential item functioning tests
 #'
 #' This function runs the Wald and likelihood-ratio approaches for testing differential
-#' item functioning (DIF). This is primarily a convenience wrapper to the \code{\link{multipleGroup}}
-#' function for performing standard DIF procedures. Models can be estimated in
-#' parallel automatically by defining a parallel object with \code{\link{mirtCluster}}, which will help
-#' to decrease the runtime.
+#' item functioning (DIF). This is primarily a convenience wrapper to the 
+#' \code{\link{multipleGroup}} function for performing standard DIF procedures. Models can be 
+#' estimated in parallel automatically by defining a parallel object with \code{\link{mirtCluster}}, 
+#' which will help to decrease the runtime.
 #'
 #' @aliases DIF
-#' @param MGmodel an object returned from \code{\link{multipleGroup}} to be used as the reference model
-#' @param which.par a character vector containing the parameter names which will be inspected for DIF
+#' @param MGmodel an object returned from \code{\link{multipleGroup}} to be used as the reference 
+#'   model
+#' @param which.par a character vector containing the parameter names which will be inspected for 
+#'   DIF
 #' @param Wald logical; perform Wald tests for DIF instead of likelihood ratio test?
-#' @param items2test a numeric vector, or character vector containing the item names, indicating which items
-#'   will be tested for DIF. In models where anchor items are known, omit them from this vector. For example,
-#'   if items 1 and 2 are anchors in a 10 item test, then \code{items2test = 3:10} would work for testing the
-#'   remaining items (important to remember when using sequential schemes)
-#' @param scheme type of DIF analysis to perform, either by adding or dropping constraints across groups.
-#'   These can be:
+#' @param items2test a numeric vector, or character vector containing the item names, indicating 
+#'   which items will be tested for DIF. In models where anchor items are known, omit them from 
+#'   this vector. For example, if items 1 and 2 are anchors in a 10 item test, then 
+#'   \code{items2test = 3:10} would work for testing the remaining items (important to remember 
+#'   when using sequential schemes)
+#' @param scheme type of DIF analysis to perform, either by adding or dropping constraints across
+#'   groups. These can be:
 #' \describe{
-#'   \item{'add'}{parameters in \code{which.par} will be constrained each item one at a time for items that are
-#'     specified in \code{items2test}. This is beneficial when examining DIF from a model with parameters
-#'     freely estimated across groups, and when inspecting differences via the Wald test}
+#'   \item{'add'}{parameters in \code{which.par} will be constrained each item one at a time for 
+#'     items that are specified in \code{items2test}. This is beneficial when examining DIF from a 
+#'     model with parameters freely estimated across groups, and when inspecting differences via 
+#'     the Wald test}
 #'   \item{'drop'}{parameters in \code{which.par} will be freely estimated for items that are
-#'     specified in \code{items2test}. This is useful when supplying an overly restrictive model and attempting to
-#'     detect DIF with a slightly less restrictive model}
-#'   \item{'add_sequential'}{sequentially loop over the items being tested, and at the end of the loop treat
-#'     DIF tests that satisfy the \code{seq_stat} criteria as invariant. The loop is then re-run on the remaining
-#'     invariant items to determine if they are now displaying DIF in the less constrained model,
-#'     and when no new invariant item is found the algorithm stops and returns the items that displayed DIF}
-#'   \item{'drop_sequential'}{sequentially loop over the items being tested, and at the end of the loop treat
-#'     items that violate the \code{seq_stat} criteria as demonstrating DIF. The loop is then re-run, leaving the items
-#'     that previously demonstrated DIF as variable across groups, and the remaining test items that previously showed
-#'     invariance are re-tested. The algorithm stops when no more items showing DIF are found and returns the items that
-#'     displayed DIF}
+#'     specified in \code{items2test}. This is useful when supplying an overly restrictive model 
+#'     and attempting to detect DIF with a slightly less restrictive model}
+#'   \item{'add_sequential'}{sequentially loop over the items being tested, and at the end of the 
+#'     loop treat DIF tests that satisfy the \code{seq_stat} criteria as invariant. The loop is 
+#'     then re-run on the remaining invariant items to determine if they are now displaying DIF in 
+#'     the less constrained model, and when no new invariant item is found the algorithm stops and 
+#'     returns the items that displayed DIF}
+#'   \item{'drop_sequential'}{sequentially loop over the items being tested, and at the end of the 
+#'     loop treat items that violate the \code{seq_stat} criteria as demonstrating DIF. The loop is 
+#'     then re-run, leaving the items that previously demonstrated DIF as variable across groups, 
+#'     and the remaining test items that previously showed invariance are re-tested. The algorithm 
+#'     stops when no more items showing DIF are found and returns the items that displayed DIF}
 #' }
 #' @param seq_stat select a statistic to test for in the sequential schemes. Potential values are
 #'   (in descending order of power) \code{'AIC'}, \code{'AICc'}, \code{'SABIC'}, and \code{'BIC'}.
 #'   If a numeric value is input that ranges between 0 and 1, the 'p' value will be tested
 #'   (e.g., \code{seq_stat = .05} will test for the difference of p < .05 in the add scheme,
 #'   or p > .05 in the drop scheme), along with the specified \code{p.adjust} input
-#' @param max_run a number indicating the maximum number of cycles to perform in sequential searches.
-#'   The default is to perform search until no further DIF is found
+#' @param max_run a number indicating the maximum number of cycles to perform in sequential 
+#'   searches. The default is to perform search until no further DIF is found
 #' @param plotdif logical; create itemplots for items that are displaying DIF according to the
 #'   \code{seq_stat} criteria? Only available for 'add' type schemes
 #' @param type the \code{type} of plot argument passed to \code{plot()}. Default is 'trace', though
-#'   another good option is 'infotrace'. For ease of viewing, the \code{facet_item} argument to mirt's
-#'   \code{plot()} function is set to \code{TRUE}
+#'   another good option is 'infotrace'. For ease of viewing, the \code{facet_item} argument to 
+#'   mirt's \code{plot()} function is set to \code{TRUE}
 #' @param p.adjust string to be passed to the \code{\link{p.adjust}} function to adjust p-values.
 #'   Adjustments are located in the \code{adj_pvals} element in the returned list
 #' @param verbose logical print extra information to the console?
@@ -119,9 +124,8 @@
 #' model <- multipleGroup(dat, 1, group, invariance = itemnames)
 #' stepdown <- DIF(model, c('a1', 'd'), scheme = 'drop_sequential')
 #' stepdown
-#'
 #' }
-DIF <- function(MGmodel, which.par, scheme = 'add', items2test = 1:ncol(MGmodel@data),
+DIF <- function(MGmodel, which.par, scheme = 'add', items2test = 1:ncol(MGmodel@Data$data),
                 seq_stat = 'SABIC', Wald = FALSE, p.adjust = 'none', max_run = Inf,
                 plotdif = FALSE, type = 'trace', verbose = TRUE, ...){
 
@@ -134,7 +138,8 @@ DIF <- function(MGmodel, which.par, scheme = 'add', items2test = 1:ncol(MGmodel@
                                              values$item == itemnames[item]]
         if(!length(parnum))
             stop('Item ', item, ' does not contain any of the parameters defined in which.par.
-                 Consider removing it from the item2test input or adding relevant parameters to which.par')
+                 Consider removing it from the item2test input or adding relevant parameters 
+                 to which.par')
         if(Wald){
             wv <- wald(model)
             infoname <- wv[1L, ]
@@ -157,7 +162,7 @@ DIF <- function(MGmodel, which.par, scheme = 'add', items2test = 1:ncol(MGmodel@
             for(i in 1L:length(parnum))
                 constrain[[length(constrain) + 1L]] <- parnum[[i]]
         }
-        newmodel <- multipleGroup(model@data, model@model[[1L]], group=model@group,
+        newmodel <- multipleGroup(model@Data$data, model@model[[1L]], group=model@Data$group,
                                   invariance = invariance, constrain=constrain,
                                   verbose = FALSE, ...)
         aov <- anova(newmodel, model, verbose = FALSE)
@@ -165,7 +170,7 @@ DIF <- function(MGmodel, which.par, scheme = 'add', items2test = 1:ncol(MGmodel@
         return(aov)
     }
 
-    itemnames <- colnames(MGmodel@data)
+    itemnames <- colnames(MGmodel@Data$data)
     if(!any(scheme %in% c('add', 'drop', 'add_sequential', 'drop_sequential')))
         stop('scheme input is not valid')
     if(Wald){
@@ -182,11 +187,12 @@ DIF <- function(MGmodel, which.par, scheme = 'add', items2test = 1:ncol(MGmodel@
         stop('Invalid seq_stat input')
     }
     if(is.character(items2test)) items2test <- which(items2test %in% itemnames)
-    data <- MGmodel@data
+    data <- MGmodel@Data$data
     invariance <- MGmodel@invariance
     values <- mod2values(MGmodel)
     drop <- scheme == 'drop' || scheme == 'drop_sequential'
-    invariance <- MGmodel@invariance[MGmodel@invariance %in% c('free_means', 'free_var', 'free_varcov', 'free_cov')]
+    invariance <- MGmodel@invariance[MGmodel@invariance %in% 
+                                         c('free_means', 'free_var', 'free_varcov', 'free_cov')]
     if(!length(invariance)) invariance <- ''
     res <- myLapply(X=items2test, FUN=loop_test, model=MGmodel, which.par=which.par, values=values, 
                     Wald=Wald, drop=drop, itemnames=itemnames, invariance=invariance, ...)
@@ -211,7 +217,8 @@ DIF <- function(MGmodel, which.par, scheme = 'add', items2test = 1:ncol(MGmodel@
             if(all(keep == lastkeep)) break
             lastkeep <- keep
             if(verbose)
-                cat(sprintf('\rChecking for DIF in %d more items', ifelse(drop, sum(keep), sum(!keep))))
+                cat(sprintf('\rChecking for DIF in %d more items', 
+                            ifelse(drop, sum(keep), sum(!keep))))
             constrain <- updatedModel@constrain
             for(j in 1L:length(keep)){
                 parnum <- list()
@@ -230,7 +237,8 @@ DIF <- function(MGmodel, which.par, scheme = 'add', items2test = 1:ncol(MGmodel@
                     }
                 }
             }
-            updatedModel <- multipleGroup(MGmodel@data, MGmodel@model[[1L]], group=MGmodel@group,
+            updatedModel <- multipleGroup(MGmodel@Data$data, MGmodel@model[[1L]], 
+                                          group=MGmodel@Data$group,
                                           invariance = invariance, constrain=constrain,
                                           verbose = FALSE, ...)
             pick <- !keep
