@@ -21,6 +21,9 @@
 #' Note: for multiple group two-tier analyses only the second-tier means and variances
 #' should be freed since the specific factors are not treated independently due to the
 #' dimension reduction technique.
+#' 
+#' @return function returns an object of class \code{ConfirmatoryClass} 
+#'   (\link{ConfirmatoryClass-class}).
 #'
 #' @aliases bfactor
 #' @param data a \code{matrix} or \code{data.frame} that consists of
@@ -31,27 +34,17 @@
 #'   on the last two, then the vector is \code{c(1,1,2,2)}. For items that should only load
 #'   on the second-tier factors (have no specific component) \code{NA} values may
 #'   be used as place-holders. These numbers will be translated into a format suitable for
-#'   \code{mirt.model()}, combined with the definition in \code{model2}, with the letter 'S' added to the
-#'   respective factor number
+#'   \code{mirt.model()}, combined with the definition in \code{model2}, with the letter 'S' 
+#'   added to the respective factor number
 #' @param model2 a two-tier model specification object defined by \code{mirt.model()}. By default
 #'   the model will fit a unidimensional model in the second-tier, and therefore be equivalent to
 #'   the bifactor model
 #' @param group a factor variable indicating group membership used for multiple group analyses
-#' @param SE logical; calculate information matrix and standard errors?
-#' @param SE.type type of standard errors to calculate. See \code{\link{mirt}} for details
-#' @param verbose logical; print observed log-likelihood value at each iteration?
-#' @param ... additional arguments to be passed to the main estimation function. See \code{\link{mirt}}
-#'   for more details
+#' @param quadpts number of quadrature nodes to use. Default is 21
+#' @param ... additional arguments to be passed to the estimation engine. See \code{\link{mirt}}
+#'   for more details and examples
 #' @author Phil Chalmers \email{rphilip.chalmers@@gmail.com}
-#' @seealso \code{\link{anova-method}}, \code{\link{coef-method}}, \code{\link{summary-method}},
-#'   \code{\link{residuals-method}}, \code{\link{plot-method}},
-#'   \code{\link{expand.table}}, \code{\link{key2binary}}, \code{\link{mirt.model}}, \code{\link{mirt}},
-#'   \code{\link{bfactor}}, \code{\link{multipleGroup}}, \code{\link{mixedmirt}},
-#'   \code{\link{wald}}, \code{\link{itemplot}}, \code{\link{fscores}},
-#'   \code{\link{M2}},
-#'   \code{\link{extract.item}}, \code{\link{iteminfo}}, \code{\link{testinfo}}, \code{\link{probtrace}},
-#'   \code{\link{boot.mirt}}, \code{\link{imputeMissing}}, \code{\link{itemfit}}, \code{\link{mod2values}},
-#'   \code{\link{simdata}}, \code{\link{createItem}}
+#' @seealso \code{\link{mirt}}
 #' @references
 #'
 #' Cai, L. (2010). A two-tier full-information item factor analysis model with applications.
@@ -190,7 +183,7 @@
 #'     }
 #'
 bfactor <- function(data, model, model2 = mirt.model(paste0('G = 1-', ncol(data))),
-                    SE = FALSE, SE.type = 'crossprod', group = NULL, verbose = TRUE, ...)
+                    group = NULL, quadpts = 21, ...)
 {
     Call <- match.call()
     if(!is.numeric(model))
@@ -208,8 +201,8 @@ bfactor <- function(data, model, model2 = mirt.model(paste0('G = 1-', ncol(data)
     attr(model, 'specific') <- specific
     if(is.null(group)) group <- rep('all', nrow(data))
     mod <- ESTIMATION(data=data, model=model, group=group,
-                      method = 'EM', verbose=verbose,
-                      BFACTOR = TRUE, SE=SE, SE.type=SE.type, ...)
+                      method = 'EM', quadpts=quadpts,
+                      BFACTOR = TRUE, ...)
     if(is(mod, 'ConfirmatoryClass') || is(mod, 'MultipleGroupClass'))
         mod@Call <- Call
     return(mod)
