@@ -5,23 +5,25 @@
 #'
 #'
 #' @aliases itemplot
-#' @param object a computed model object of class \code{ExploratoryClass}, \code{ConfirmatoryClass}, or
-#'   \code{MultipleGroupClass}. Input may also be a \code{list} for comparing similar item types (e.g., 1PL vs 2PL)
+#' @param object a computed model object of class \code{ExploratoryClass}, \code{ConfirmatoryClass},
+#'   or \code{MultipleGroupClass}. Input may also be a \code{list} for comparing similar item types
+#'   (e.g., 1PL vs 2PL)
 #' @param item a single numeric value, or the item name, indicating which item to plot
 #' @param type plot type to use, information (\code{'info'}), standard errors (\code{'SE'}), 
 #'   item trace lines (\code{'trace'}), information and standard errors (\code{'infoSE'}) or 
 #'   information and trace lines (\code{'infotrace'}), relative efficiency lines (\code{'RE'}), 
 #'   expected score \code{'score'}, or information and trace line contours (\code{'infocontour'} and 
 #'   \code{'tracecontour'}; not supported for \code{MultipleGroupClass} objects)
-#' @param degrees the degrees argument to be used if there are exactly two factors. See \code{\link{iteminfo}}
-#'   for more detail
+#' @param degrees the degrees argument to be used if there are exactly two factors. 
+#'   See \code{\link{iteminfo}} for more detail
 #' @param CE logical; plot confidence envelope?
-#' @param CEalpha area remaining in the tail for confidence envelope. Default gives 95\% confidence region
+#' @param CEalpha area remaining in the tail for confidence envelope. Default gives 95\% confidence
+#'   region
 #' @param CEdraws draws number of draws to use for confidence envelope
 #' @param rot a list of rotation coordinates to be used for 3 dimensional plots
-#' @param drop.zeros logical; drop slope values that are numerically close to zero to reduce dimensionality?
-#'   Useful in objects returned from \code{\link{bfactor}} or other confirmatory models that contain several
-#'   zero slopes
+#' @param drop.zeros logical; drop slope values that are numerically close to zero to reduce 
+#'   dimensionality? Useful in objects returned from \code{\link{bfactor}} or other confirmatory
+#'   models that contain several zero slopes
 #' @param theta_lim lower and upper limits of the latent trait (theta) to be evaluated, and is 
 #'   used in conjunction with \code{npts}
 #' @param shiny logical; run interactive display for item plots using the \code{shiny} interface.
@@ -65,14 +67,15 @@
 #'     }
 #'
 itemplot <- function(object, item, type = 'trace', degrees = 45, CE = FALSE, CEalpha = .05,
-                     CEdraws = 1000, drop.zeros = FALSE, rot = list(xaxis = -70, yaxis = 30, zaxis = 10),
-                     theta_lim = c(-4,4), shiny = FALSE, ...){
+                     CEdraws = 1000, drop.zeros = FALSE, 
+                     rot = list(xaxis = -70, yaxis = 30, zaxis = 10),
+                     theta_lim = c(-6,6), shiny = FALSE, ...){
     if(shiny){
         require(shiny)
         runApp(shinyItemplot())
     }
-    if(is.list(object)) inames <- colnames(object[[1]]@data)
-    else inames <- colnames(object@data)
+    if(is.list(object)) inames <- colnames(object[[1]]@Data$data)
+    else inames <- colnames(object@Data$data)
     ind <- 1:length(inames)
     if(!is.numeric(item)) item <- ind[inames == item]
     rot <- list(x = rot[[1]], y = rot[[2]], z = rot[[3]])
