@@ -16,21 +16,20 @@ test_that('exploratory mods', {
     L2[1, 7] <- L2[2, 9] <- -1
     W1 <- wald(onefact, L)
     W2 <- wald(onefact, L2)
-    expect_is(W1, 'wald')
-    expect_is(W2, 'wald')
     expect_true(mirt:::closeEnough(W1$W - 212.3752, -1e-2, 1e-2))
     expect_true(mirt:::closeEnough(W2$W - 2.080472, -1e-2, 1e-2))    
     
     fitonefact <- M2(onefact)
     expect_is(fitonefact, 'data.frame')
     expect_equal(fitonefact$M2, 11.92959, tolerance = 1e-2)
-    suppressWarnings(twofact <- mirt(fulldata, 2, verbose = FALSE, draws = 10, method = 'MHRM'))
+    twofact <- mirt(fulldata, 2, verbose = FALSE, draws = 10, method = 'MHRM')
     cfs <- as.numeric(do.call(c, coef(twofact, digits=4, verbose = FALSE)))
     expect_equal(cfs, c(0.0023, -0.4689, 0.4736, -1.3362, -2.151, -0.5215, 2.0565, 1.7901, 2.3229, 0, NA, NA, 1, NA, NA, 1.8471, 1.6171, 2.0771, 0.1089, -0.5237, 0.7415, 1.0121, 0.7786, 1.2457, 0, NA, NA, 1, NA, NA, 1.0655, 0.7165, 1.4146, -0.6646, -0.9128, -0.4165, 1.7122, 1.4201, 2.0042, 0, NA, NA, 1, NA, NA, 0.185, -0.1054, 0.4755, -0.7199, -1.2577, -0.1821, 0.4962, 0.3554, 0.637, 0, NA, NA, 1, NA, NA, -0.0826, -0.233, 0.0679, -0.9982, -1.0461, -0.9504, 1.9631, 1.7076, 2.2187, 0, NA, NA, 1, NA, NA, 0, NA, NA, 0, NA, NA, 1, NA, NA, -0.5326, NA, NA, 1, NA, NA),
                  tolerance = 1e-2)
     expect_is(twofact, 'ExploratoryClass')
-    modm7 <- suppressWarnings(suppressMessages(mirt(fulldata, 1, '4PL', verbose=FALSE, parprior = list(c(3,7,11,15,19,'norm', -1.7, 1),
-                                                                     c(4,8,12,16,20,'norm', 1.7, 1)), method = 'MHRM', draws = 10)))
+    expect_message(modm7 <- mirt(fulldata, 1, '4PL', verbose=FALSE, parprior = list(c(3,7,11,15,19,'norm', -1.7, 1),
+                                                                     c(4,8,12,16,20,'norm', 1.7, 1)), method = 'MHRM', draws = 10),
+                            "MHRM terminated after 2000 iterations.")
     expect_equal(modm7@df, 11)
     expect_is(modm7, 'ConfirmatoryClass')
     cfs <- as.numeric(do.call(c, coef(modm7)))
@@ -42,10 +41,9 @@ test_that('exploratory mods', {
     expect_equal(cfs, c(0.9863, 0.6452, 1.3274, 1.8592, 1.6216, 2.0967, 0, NA, NA, 1, NA, NA, 1.0755, 0.7674, 1.3836, 0.8079, 0.6293, 0.9866, 0, NA, NA, 1, NA, NA, 1.6688, 1.1644, 2.1731, 1.7831, 1.4462, 2.12, 0, NA, NA, 1, NA, NA, 0.7533, 0.5088, 0.9978, 0.4845, 0.3287, 0.6402, 0, NA, NA, 1, NA, NA, 0.7705, 0.4204, 1.1206, 1.8698, 1.6072, 2.1324, 0, NA, NA, 1, NA, NA, 0, NA, NA, 1, NA, NA),
                  tolerance = 1e-2)
 
-
     fs1 <- fscores(onefact, verbose = FALSE, mean=c(1), cov=matrix(2))
     expect_is(fs1, 'matrix')
-    expect_true(mirt:::closeEnough(fs1[1:3,'F1'] - c(-2.148334, -1.681619, -1.687547), -1e-2, 1e-2))
+    expect_true(mirt:::closeEnough(fs1[1:3,'F1'] - c(-2.185196, -1.689718, -1.694422), -1e-2, 1e-2))
     fs2 <- fscores(twofact, verbose = FALSE)
     expect_is(fs2, 'matrix')
     fs3 <- fscores(onefactmissing, verbose = FALSE)
